@@ -1,13 +1,20 @@
-/* Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -99,7 +106,7 @@ Format_description_event::Format_description_event(uint8_t binlog_ver,
      in getting rid of the valgrind errors.
     */
     memset(server_version, 0, ST_SERVER_VER_LEN);
-    strncpy(server_version, server_ver, ST_SERVER_VER_LEN);
+    strncpy(server_version, server_ver, ST_SERVER_VER_LEN - 1);
     if (binary_log_debug::debug_pretend_version_50034_in_binlog)
       strcpy(server_version, "5.0.34");
     common_header_len= LOG_EVENT_HEADER_LEN;
@@ -163,8 +170,6 @@ Format_description_event::Format_description_event(uint8_t binlog_ver,
        Allows us to sanity-check that all events initialized their
        events (see the end of this 'if' block).
     */
-    post_header_len.resize(number_of_event_types +
-                            BINLOG_CHECKSUM_ALG_DESC_LEN, 255);
     post_header_len.insert(post_header_len.begin(), server_event_header_length,
                             server_event_header_length + number_of_event_types);
     // Sanity-check that all post header lengths are initialized.
@@ -220,7 +225,6 @@ Format_description_event::Format_description_event(uint8_t binlog_ver,
       RAND_HEADER_LEN,
       USER_VAR_HEADER_LEN
     };
-    post_header_len.resize(number_of_event_types + BINLOG_CHECKSUM_ALG_DESC_LEN);
     post_header_len.insert(post_header_len.begin(),
                            server_event_header_length_ver_1_3,
                            server_event_header_length_ver_1_3 +
@@ -338,18 +342,21 @@ Format_description_event(const char* buf, unsigned int event_len,
   number_of_event_types=
    event_len - (LOG_EVENT_MINIMAL_HEADER_LEN + ST_COMMON_HEADER_LEN_OFFSET + 1);
 
-  const uint8_t *ubuf = reinterpret_cast<const uint8_t*>(buf);
-  post_header_len.resize(number_of_event_types);
-  post_header_len.insert(post_header_len.begin(),
-                         ubuf + ST_COMMON_HEADER_LEN_OFFSET + 1,
-                         (ubuf + ST_COMMON_HEADER_LEN_OFFSET + 1 +
-                          number_of_event_types));
-
   calc_server_version_split();
-  if ((ver_calc= get_product_version()) >= checksum_version_product)
-  {
+  ver_calc = get_product_version();
+  if (ver_calc >= checksum_version_product) {
     /* the last bytes are the checksum alg desc and value (or value's room) */
     number_of_event_types -= BINLOG_CHECKSUM_ALG_DESC_LEN;
+  }
+
+  const uint8_t *ubuf = reinterpret_cast<const uint8_t*>(buf) +
+                        ST_COMMON_HEADER_LEN_OFFSET + 1;
+  post_header_len.insert(post_header_len.begin(),
+                         ubuf, (ubuf + number_of_event_types));
+
+  ubuf += number_of_event_types;
+  if (ver_calc >= checksum_version_product)
+  {
     /*
       FD from the checksum-home version server (ver_calc ==
       checksum_version_product) must have
@@ -357,8 +364,7 @@ Format_description_event(const char* buf, unsigned int event_len,
     */
     BAPI_ASSERT(ver_calc != checksum_version_product ||
                 number_of_event_types == LOG_EVENT_TYPES);
-    footer()->checksum_alg= (enum_binlog_checksum_alg)
-                                  post_header_len[number_of_event_types];
+    footer()->checksum_alg= static_cast<enum_binlog_checksum_alg> (*ubuf);
   }
   else
   {
@@ -606,7 +612,20 @@ XA_prepare_event(const char* buf,
   memcpy(&temp, buf, sizeof(temp));
   my_xid.bqual_length= le32toh(temp);
   buf += sizeof(temp);
-  memcpy(my_xid.data, buf, my_xid.gtrid_length + my_xid.bqual_length);
+
+  /* Sanity check */
+  if (MY_XIDDATASIZE >= my_xid.gtrid_length + my_xid.bqual_length &&
+      my_xid.gtrid_length >=0 && my_xid.gtrid_length <= 64 &&
+      my_xid.bqual_length >=0 && my_xid.bqual_length <= 64)
+  {
+    memcpy(my_xid.data, buf, my_xid.gtrid_length + my_xid.bqual_length);
+  }
+  else
+  {
+    my_xid.formatID= -1;
+    my_xid.gtrid_length= 0;
+    my_xid.bqual_length= 0;
+  }
 }
 
 /**
@@ -715,6 +734,16 @@ Transaction_context_event(const char *buffer, unsigned int event_len,
   //buf is advanced in Binary_log_event constructor to point to
   //beginning of post-header
   const char* data_head = buffer;
+  const char* buffer_start = buffer - description_event->common_header_len;
+
+  server_uuid= NULL;
+  encoded_snapshot_version= NULL;
+
+  /* Avoid reading out of buffer */
+  if (static_cast<unsigned int>(description_event->common_header_len +
+                                TRANSACTION_CONTEXT_HEADER_LEN) > event_len)
+    return;
+
   uint8_t server_uuid_len= (static_cast<unsigned int>
                            (data_head[ENCODED_SERVER_UUID_LEN_OFFSET]));
 
@@ -739,6 +768,14 @@ Transaction_context_event(const char *buffer, unsigned int event_len,
   gtid_specified= (int8_t) data_head[ENCODED_GTID_SPECIFIED_OFFSET];
 
   const char *pos = data_head + TRANSACTION_CONTEXT_HEADER_LEN;
+  uint32_t remaining_buffer= 0;
+
+  /* Avoid reading out of buffer */
+  if (event_len < (TRANSACTION_CONTEXT_HEADER_LEN + server_uuid_len +
+                   encoded_snapshot_version_length +
+                   write_set_len + read_set_len))
+    goto err;
+
   server_uuid= bapi_strndup(pos, server_uuid_len);
   pos += server_uuid_len;
 
@@ -746,20 +783,24 @@ Transaction_context_event(const char *buffer, unsigned int event_len,
       reinterpret_cast<const unsigned char *>(bapi_strndup(pos,
                                                            encoded_snapshot_version_length));
   pos += encoded_snapshot_version_length;
+  remaining_buffer= event_len - (pos - buffer_start);
 
-  pos= read_data_set(pos, write_set_len, &write_set);
+  pos= read_data_set(pos, write_set_len, &write_set, remaining_buffer);
   if (pos == NULL)
     goto err;
-  pos= read_data_set(pos, read_set_len, &read_set);
+  remaining_buffer= event_len - (pos - buffer_start);
+  pos= read_data_set(pos, read_set_len, &read_set, remaining_buffer);
   if (pos == NULL)
     goto err;
 
   return;
 
 err:
-  bapi_free((void*)server_uuid);
+  if (server_uuid)
+    bapi_free((void*)server_uuid);
   server_uuid= NULL;
-  bapi_free((void*)encoded_snapshot_version);
+  if (encoded_snapshot_version)
+    bapi_free((void*)encoded_snapshot_version);
   encoded_snapshot_version= NULL;
   clear_set(&write_set);
   clear_set(&read_set);
@@ -769,27 +810,35 @@ err:
 /**
   Function to read the data set for the ongoing transaction.
 
-  @param[in] pos     - postion to read from.
-  @param[in] set_len - length of the set object
-  @param[in] set     - pointer to the set object
+  @param[in] pos       - postion to read from.
+  @param[in] set_len   - length of the set object
+  @param[in] set       - pointer to the set object
+  @param[in] remaining - remaining available bytes on the buffer
 
   @retval - returns the pointer in the buffer to the end of the added hash
-            value.
+            value or NULL in case of an error.
 */
 const char* Transaction_context_event::read_data_set(const char *pos,
                                                      uint32_t set_len,
-                                                     std::list<const char*> *set)
+                                                     std::list<const char*> *set,
+                                                     uint32_t remaining)
 {
   uint16_t len= 0;
   for (uint32_t i= 0; i < set_len; i++)
   {
+    if (remaining < static_cast<uint32_t>(ENCODED_READ_WRITE_SET_ITEM_LEN))
+      return(NULL);
     memcpy(&len, pos, 2);
     len= le16toh(len);
+    remaining-= ENCODED_READ_WRITE_SET_ITEM_LEN;
     pos += ENCODED_READ_WRITE_SET_ITEM_LEN;
+    if (remaining < len)
+      return(NULL);
     const char *hash= bapi_strndup(pos, len);
     if (hash == NULL)
       return(NULL);
     pos += len;
+    remaining-= len;
     set->push_back(hash);
   }
   return(pos);
@@ -833,7 +882,8 @@ View_change_event::View_change_event(char* raw_view_id)
 : Binary_log_event(VIEW_CHANGE_EVENT),
   view_id(), seq_number(0), certification_info()
 {
-  memcpy(view_id, raw_view_id, strlen(raw_view_id));
+  strncpy(view_id, raw_view_id, sizeof(view_id)-1);
+  view_id[sizeof(view_id)-1]= 0;
 }
 
 View_change_event::
@@ -857,7 +907,20 @@ View_change_event(const char *buffer, unsigned int event_len,
 
   char *pos = (char*) data_header + VIEW_CHANGE_HEADER_LEN;
 
-  pos= read_data_map(pos, cert_info_len, &certification_info);
+  /* Avoid reading out of buffer */
+  if (event_len < (LOG_EVENT_HEADER_LEN + VIEW_CHANGE_HEADER_LEN))
+  {
+    pos= NULL;
+  }
+  else
+  {
+    unsigned int max_cert_info_len= event_len - (LOG_EVENT_HEADER_LEN +
+                                                 VIEW_CHANGE_HEADER_LEN);
+
+    pos= read_data_map(pos, cert_info_len, &certification_info,
+                       max_cert_info_len);
+  }
+
   if (pos == NULL)
     // Make is_valid() defined in the server return false.
     view_id[0]= '\0';
@@ -871,33 +934,49 @@ View_change_event(const char *buffer, unsigned int event_len,
   @param[in] pos     - start position.
   @param[in] map_len - the length of the certification info map.
   @param[in] map     - Certification info map
+  @param[in] consumable - the amount of bytes that can be read from buffer
 
   @return pointer to the snapshot version.
 */
 char* View_change_event::read_data_map(char *pos,
                                        uint32_t map_len,
-                                       std::map<std::string, std::string> *map)
+                                       std::map<std::string, std::string> *map,
+                                       uint32_t consumable)
 {
   BAPI_ASSERT(map->empty());
   uint16_t created= 0;
   uint32_t created_value= 0;
   for (uint32_t i= 0; i < map_len; i++)
   {
+    if (!consumable ||
+        consumable < static_cast<uint32_t>(ENCODED_CERT_INFO_KEY_SIZE_LEN))
+      return NULL;
     created=0;
     memcpy(&created, pos, sizeof(created));
     uint16_t key_len= (uint16_t) le16toh(created);
     pos+= ENCODED_CERT_INFO_KEY_SIZE_LEN;
+    consumable-= ENCODED_CERT_INFO_KEY_SIZE_LEN;
 
+    if (!consumable || consumable < key_len)
+      return NULL;
     std::string key(pos, key_len);
     pos+= key_len;
+    consumable-= key_len;
 
+    if (!consumable ||
+        consumable < static_cast<uint32_t>(ENCODED_CERT_INFO_VALUE_LEN))
+      return NULL;
     created_value=0;
     memcpy(&created_value, pos, sizeof(created_value));
     uint32_t value_len= le32toh(created_value);
     pos+= ENCODED_CERT_INFO_VALUE_LEN;
+    consumable-= ENCODED_CERT_INFO_VALUE_LEN;
 
+    if (!consumable || consumable < value_len)
+      return NULL;
     std::string value(pos, value_len);
     pos+= value_len;
+    consumable-= value_len;
 
     (*map)[key]= value;
   }
