@@ -33,6 +33,7 @@ our @EXPORT= qw(collect_option collect_test_cases);
 
 use mtr_report;
 use mtr_match;
+use My::Constants;
 
 # Options used for the collect phase
 our $start_from;
@@ -693,6 +694,7 @@ sub optimize_cases {
 	if ( !$supported )
 	{
 	  $tinfo->{'skip'}= 1;
+	  $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
 	  $tinfo->{'comment'}=
 	    "Doesn't support --binlog-format='$binlog_format'";
 	}
@@ -721,6 +723,7 @@ sub optimize_cases {
 	if ( !$supported )
 	{
 	  $tinfo->{'skip'}= 1;
+	  $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
 	  $tinfo->{'comment'}=
 	    "Doesn't support --binlog-format='$test_binlog_format'";
 	  next;
@@ -757,6 +760,7 @@ sub optimize_cases {
 	     ! exists $builtin_engines{$default_engine} )
 	{
 	  $tinfo->{'skip'}= 1;
+	  $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
 	  $tinfo->{'comment'}=
 	    "'$default_engine' not supported";
 	}
@@ -778,6 +782,7 @@ sub optimize_cases {
 	     ! exists $builtin_engines{$default_tmp_engine} )
 	{
 	  $tinfo->{'skip'}= 1;
+	  $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
 	  $tinfo->{'comment'}=
 	    "'$default_tmp_engine' not supported";
 	}
@@ -1022,6 +1027,7 @@ sub collect_one_test_case {
     if ( IS_WIN32PERL )
     {
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "No tests with sh scripts on Windows";
       return $tinfo;
     }
@@ -1040,6 +1046,7 @@ sub collect_one_test_case {
     if ( IS_WIN32PERL )
     {
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "No tests with sh scripts on Windows";
       return $tinfo;
     }
@@ -1073,6 +1080,7 @@ sub collect_one_test_case {
   if ( $tinfo->{'big_test'} and ! $::opt_big_test )
   {
     $tinfo->{'skip'}= 1;
+    $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
     $tinfo->{'comment'}= "Test needs 'big-test' option";
     return $tinfo
   }
@@ -1080,6 +1088,7 @@ sub collect_one_test_case {
   if ( $tinfo->{'need_debug'} && ! $::debug_compiled_binaries )
   {
     $tinfo->{'skip'}= 1;
+    $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
     $tinfo->{'comment'}= "Test needs debug binaries";
     return $tinfo
   }
@@ -1091,6 +1100,7 @@ sub collect_one_test_case {
     {
       # ndbcluster is disabled
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "ndbcluster disabled";
       return $tinfo;
     }
@@ -1102,6 +1112,7 @@ sub collect_one_test_case {
     {
       # Only the ndb test should be run, all other should be skipped
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "Only ndbcluster tests";
       return $tinfo;
     }
@@ -1127,6 +1138,7 @@ sub collect_one_test_case {
     if (grep(/^--skip[-_]log[-_]bin/,  @::opt_extra_mysqld_opt) )
     {
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "Test needs binlog";
       return $tinfo;
     }
@@ -1144,6 +1156,7 @@ sub collect_one_test_case {
     if ( $skip_rpl )
     {
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "No replication tests(--skip-rpl)";
       return $tinfo;
     }
@@ -1154,6 +1167,7 @@ sub collect_one_test_case {
     if ( $tinfo->{'not_embedded'} )
     {
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "Not run for embedded server";
       return $tinfo;
     }
@@ -1173,6 +1187,7 @@ sub collect_one_test_case {
     if ( ! $::opt_ssl_supported ) {
       # SSL is not supported, skip it
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "No SSL support";
       return $tinfo;
     }
@@ -1193,6 +1208,7 @@ sub collect_one_test_case {
   if ( $tinfo->{'not_windows'} && IS_WINDOWS )
   {
     $tinfo->{'skip'}= 1;
+    $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
     $tinfo->{'comment'}= "Test not supported on Windows";
     return $tinfo;
   }
