@@ -1,6 +1,6 @@
 /***********************************************************************
 
-Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -57,23 +57,14 @@ cb_cursor_moveto_t		ib_cb_moveto;
 cb_sec_search_tuple_create_t	ib_cb_search_tuple_create;
 cb_sec_read_tuple_create_t	ib_cb_read_tuple_create;
 cb_tuple_delete_t		ib_cb_tuple_delete;
-cb_tuple_copy_t			ib_cb_tuple_copy;
 cb_tuple_read_u8_t		ib_cb_tuple_read_u8;
 cb_tuple_read_u16_t		ib_cb_tuple_read_u16;
 cb_tuple_read_u32_t		ib_cb_tuple_read_u32;
 cb_tuple_read_u64_t		ib_cb_tuple_read_u64;
-cb_tuple_write_u8_t		ib_cb_tuple_write_u8;
-cb_tuple_write_u16_t		ib_cb_tuple_write_u16;
-cb_tuple_write_u32_t		ib_cb_tuple_write_u32;
-cb_tuple_write_u64_t		ib_cb_tuple_write_u64;
 cb_tuple_read_i8_t		ib_cb_tuple_read_i8;
 cb_tuple_read_i16_t		ib_cb_tuple_read_i16;
 cb_tuple_read_i32_t		ib_cb_tuple_read_i32;
 cb_tuple_read_i64_t		ib_cb_tuple_read_i64;
-cb_tuple_write_i8_t		ib_cb_tuple_write_i8;
-cb_tuple_write_i16_t		ib_cb_tuple_write_i16;
-cb_tuple_write_i32_t		ib_cb_tuple_write_i32;
-cb_tuple_write_i64_t		ib_cb_tuple_write_i64;
 cb_col_set_value_t		ib_cb_col_set_value;
 cb_col_get_value_t		ib_cb_col_get_value;
 cb_col_get_meta_t		ib_cb_col_get_meta;
@@ -81,21 +72,18 @@ cb_trx_begin_t			ib_cb_trx_begin;
 cb_trx_commit_t			ib_cb_trx_commit;
 cb_trx_rollback_t		ib_cb_trx_rollback;
 cb_trx_start_t			ib_cb_trx_start;
-cb_trx_state_t			ib_cb_trx_state;
 cb_trx_release_t		ib_cb_trx_release;
 cb_tuple_get_n_cols_t		ib_cb_tuple_get_n_cols;
 cb_cursor_set_match_mode_t	ib_cb_cursor_set_match_mode;
 cb_cursor_lock_t		ib_cb_cursor_lock;
 cb_cursor_close_t		ib_cb_cursor_close;
 cb_cursor_new_trx_t		ib_cb_cursor_new_trx;
-cb_open_table_by_name_t		ib_cb_open_table_by_name;
 cb_cursor_reset_t		ib_cb_cursor_reset;
 cb_col_get_name_t		ib_cb_col_get_name;
 cb_get_idx_field_name		ib_cb_get_idx_field_name;
 cb_table_truncate_t		ib_cb_table_truncate;
 cb_cursor_first_t		ib_cb_cursor_first;
 cb_cursor_next_t		ib_cb_cursor_next;
-cb_cursor_last_t		ib_cb_cursor_last;
 cb_cursor_open_index_using_name_t	ib_cb_cursor_open_index_using_name;
 cb_close_thd_t			ib_cb_close_thd;
 cb_get_cfg_t			ib_cb_get_cfg;
@@ -105,11 +93,12 @@ cb_cursor_commit_trx_t		ib_cb_cursor_commit_trx;
 cb_cfg_trx_level_t		ib_cb_cfg_trx_level;
 cb_get_n_user_cols		ib_cb_get_n_user_cols;
 cb_cursor_set_lock_t		ib_cb_cursor_set_lock;
-cb_cursor_clear_trx_t		ib_cb_cursor_clear_trx;
 cb_trx_get_start_time		ib_cb_trx_get_start_time;
 cb_bk_commit_interval		ib_cb_cfg_bk_commit_interval;
+cb_ut_strerr			ib_cb_ut_strerr;
 cb_cursor_stmt_begin		ib_cb_cursor_stmt_begin;
 cb_trx_read_only_t		ib_cb_trx_read_only;
+cb_is_virtual_table		ib_cb_is_virtual_table;
 
 /** InnoDB API callback functions */
 static ib_cb_t* innodb_memcached_api[] = {
@@ -121,28 +110,18 @@ static ib_cb_t* innodb_memcached_api[] = {
 	(ib_cb_t*) &ib_cb_moveto,
 	(ib_cb_t*) &ib_cb_cursor_first,
 	(ib_cb_t*) &ib_cb_cursor_next,
-	(ib_cb_t*) &ib_cb_cursor_last,
 	(ib_cb_t*) &ib_cb_cursor_set_match_mode,
 	(ib_cb_t*) &ib_cb_search_tuple_create,
 	(ib_cb_t*) &ib_cb_read_tuple_create,
 	(ib_cb_t*) &ib_cb_tuple_delete,
-	(ib_cb_t*) &ib_cb_tuple_copy,
 	(ib_cb_t*) &ib_cb_tuple_read_u8,
-	(ib_cb_t*) &ib_cb_tuple_write_u8,
 	(ib_cb_t*) &ib_cb_tuple_read_u16,
-	(ib_cb_t*) &ib_cb_tuple_write_u16,
 	(ib_cb_t*) &ib_cb_tuple_read_u32,
-	(ib_cb_t*) &ib_cb_tuple_write_u32,
 	(ib_cb_t*) &ib_cb_tuple_read_u64,
-	(ib_cb_t*) &ib_cb_tuple_write_u64,
 	(ib_cb_t*) &ib_cb_tuple_read_i8,
-	(ib_cb_t*) &ib_cb_tuple_write_i8,
 	(ib_cb_t*) &ib_cb_tuple_read_i16,
-	(ib_cb_t*) &ib_cb_tuple_write_i16,
 	(ib_cb_t*) &ib_cb_tuple_read_i32,
-	(ib_cb_t*) &ib_cb_tuple_write_i32,
 	(ib_cb_t*) &ib_cb_tuple_read_i64,
-	(ib_cb_t*) &ib_cb_tuple_write_i64,
 	(ib_cb_t*) &ib_cb_tuple_get_n_cols,
 	(ib_cb_t*) &ib_cb_col_set_value,
 	(ib_cb_t*) &ib_cb_col_get_value,
@@ -152,16 +131,13 @@ static ib_cb_t* innodb_memcached_api[] = {
 	(ib_cb_t*) &ib_cb_trx_rollback,
 	(ib_cb_t*) &ib_cb_trx_start,
 	(ib_cb_t*) &ib_cb_trx_release,
-	(ib_cb_t*) &ib_cb_trx_state,
 	(ib_cb_t*) &ib_cb_cursor_lock,
 	(ib_cb_t*) &ib_cb_cursor_close,
 	(ib_cb_t*) &ib_cb_cursor_new_trx,
 	(ib_cb_t*) &ib_cb_cursor_reset,
-	(ib_cb_t*) &ib_cb_open_table_by_name,
 	(ib_cb_t*) &ib_cb_col_get_name,
 	(ib_cb_t*) &ib_cb_table_truncate,
 	(ib_cb_t*) &ib_cb_cursor_open_index_using_name,
-	(ib_cb_t*) &ib_cb_close_thd,
 	(ib_cb_t*) &ib_cb_get_cfg,
 	(ib_cb_t*) &ib_cb_cursor_set_memcached_sync,
 	(ib_cb_t*) &ib_cb_cursor_set_cluster_access,
@@ -169,12 +145,13 @@ static ib_cb_t* innodb_memcached_api[] = {
 	(ib_cb_t*) &ib_cb_cfg_trx_level,
 	(ib_cb_t*) &ib_cb_get_n_user_cols,
 	(ib_cb_t*) &ib_cb_cursor_set_lock,
-	(ib_cb_t*) &ib_cb_cursor_clear_trx,
 	(ib_cb_t*) &ib_cb_get_idx_field_name,
 	(ib_cb_t*) &ib_cb_trx_get_start_time,
 	(ib_cb_t*) &ib_cb_cfg_bk_commit_interval,
+	(ib_cb_t*) &ib_cb_ut_strerr,
 	(ib_cb_t*) &ib_cb_cursor_stmt_begin,
-	(ib_cb_t*) &ib_cb_trx_read_only
+	(ib_cb_t*) &ib_cb_trx_read_only,
+	(ib_cb_t*) &ib_cb_is_virtual_table
 };
 
 /** Set expiration time. If the exp sent by client is larger than
@@ -228,7 +205,7 @@ innodb_api_begin(
 	char		table_name[MAX_TABLE_NAME_LEN + MAX_DATABASE_NAME_LEN];
 
 	if (!*crsr) {
-#ifdef __WIN__
+#ifdef _WIN32
 		sprintf(table_name, "%s\%s", dbname, name);
 #else
 		snprintf(table_name, sizeof(table_name),
@@ -707,76 +684,6 @@ innodb_api_fill_mci(
 	return(true);
 }
 
-/*************************************************************//**
-Copy data from a read tuple and instantiate a "mci_column_t" structure
-@return true if successful */
-static
-bool
-innodb_api_copy_mci(
-/*================*/
-	ib_tpl_t	read_tpl,	/*!< in: Read tuple */
-	int		col_id,		/*!< in: Column ID for the column to
-					read */
-	mci_column_t*	mci_item)	/*!< out: item to fill */
-{
-	ib_ulint_t      data_len;
-	ib_col_meta_t   col_meta;
-
-	data_len = ib_cb_col_get_meta(read_tpl, col_id, &col_meta);
-
-	if (data_len == IB_SQL_NULL) {
-		mci_item->value_str = NULL;
-		mci_item->value_len = 0;
-		mci_item->allocated = false;
-	} else {
-		if (col_meta.type == IB_INT) {
-			mci_item->value_str = malloc(50);
-			memset(mci_item->value_str, 0, 50);
-
-			if (col_meta.attr & IB_COL_UNSIGNED) {
-				uint64_t int_val = 0;
-
-				int_val = innodb_api_read_uint64(&col_meta,
-								 read_tpl,
-								 col_id);
-
-				sprintf(mci_item->value_str,
-					"%" PRIu64, int_val);
-			} else {
-				int64_t int_val = 0;
-
-				int_val = innodb_api_read_int(&col_meta,
-							      read_tpl,
-							      col_id);
-
-				sprintf(mci_item->value_str,
-					"%" PRIi64, int_val);
-			}
-
-			mci_item->value_len = strlen(mci_item->value_str);
-			mci_item->allocated = true;
-
-		} else {
-			mci_item->value_str = malloc(data_len);
-
-			if (!mci_item->value_str) {
-				return(false);
-			}
-
-			mci_item->allocated = true;
-			memcpy(mci_item->value_str,
-			       ib_cb_col_get_value(read_tpl, col_id),
-			       data_len);
-			mci_item->value_len = data_len;
-		}
-	}
-
-	mci_item->is_str = true;
-	mci_item->is_valid = true;
-
-	return(true);
-}
-
 /*************************************************************//**
 Fetch value from a read cursor into "mci_items"
 @return DB_SUCCESS if successful */
@@ -788,8 +695,7 @@ innodb_api_fill_value(
 			meta_info,	/*!< in: Metadata */
 	mci_item_t*	item,		/*!< out: item to fill */
 	ib_tpl_t	read_tpl,	/*!< in: read tuple */
-	int		col_id,		/*!< in: column Id */
-	bool		alloc_mem)	/*!< in: allocate memory */
+	int		col_id)		/*!< in: column Id */
 {
 	ib_err_t	err = DB_NOT_FOUND;
 
@@ -800,20 +706,9 @@ innodb_api_fill_value(
 
 		if (col_id == col_info[CONTAINER_VALUE].field_id) {
 
-			if (alloc_mem) {
-
-				/* when using innodb memcache the
-				code  will never come here becasue
-				we do not allocate memory for column
-				objects */
-				innodb_api_copy_mci(
-					read_tpl, col_id,
-					&item->col_value[MCI_COL_VALUE]);
-			} else {
-				innodb_api_fill_mci(
-					read_tpl, col_id,
-					&item->col_value[MCI_COL_VALUE]);
-			}
+			innodb_api_fill_mci(
+				read_tpl, col_id,
+				&item->col_value[MCI_COL_VALUE]);
 
 			err = DB_SUCCESS;
 		}
@@ -822,15 +717,9 @@ innodb_api_fill_value(
 
 		for (i = 0; i < meta_info->n_extra_col; i++) {
 			if (col_id == meta_info->extra_col_info[i].field_id) {
-				if (alloc_mem) {
-					innodb_api_copy_mci(
-						read_tpl, col_id,
-						&item->extra_col_value[i]);
-				} else {
-					innodb_api_fill_mci(
-						read_tpl, col_id,
-						&item->extra_col_value[i]);
-				}
+				innodb_api_fill_mci(
+					read_tpl, col_id,
+					&item->extra_col_value[i]);
 
 				err = DB_SUCCESS;
 				break;
@@ -1080,7 +969,7 @@ innodb_api_search(
 			     && i == col_info[CONTAINER_VALUE].field_id)
 			    || meta_info->n_extra_col) {
 				innodb_api_fill_value(meta_info, item,
-						      read_tpl, i, false);
+						      read_tpl, i);
 			}
 		}
 
@@ -1108,7 +997,7 @@ mci_get_cas(
 {
 	static uint64_t cas_id = 0;
 
-#if defined(HAVE_GCC_ATOMIC_BUILTINS)
+#if defined(HAVE_GCC_SYNC_BUILTINS)
 	return(__sync_add_and_fetch(&cas_id, 1));
 #else
 	pthread_mutex_lock(&eng->cas_mutex);
@@ -1985,7 +1874,7 @@ innodb_api_flush(
 				   + MAX_DATABASE_NAME_LEN + 1];
 	ib_id_u64_t	new_id;
 
-#ifdef __WIN__
+#ifdef _WIN32
 	sprintf(table_name, "%s\%s", dbname, name);
 #else
 	snprintf(table_name, sizeof(table_name), "%s/%s", dbname, name);
@@ -2169,17 +2058,6 @@ innodb_cb_trx_commit(
 	return(ib_cb_trx_commit(ib_trx));
 }
 
-/*************************************************************//**
-Close table associated to the connection
-@return DB_SUCCESS if successful or error code */
-ib_err_t
-innodb_cb_close_thd(
-/*=================*/
-	void*		thd)		/*!<in: THD */
-{
-	return(ib_cb_close_thd(thd));
-}
-
 /*****************************************************************//**
 update the cursor with new transactions and also reset the cursor
 @return DB_SUCCESS or err code */
