@@ -351,6 +351,21 @@ int bootstrap(MYSQL_FILE *file)
 
   thd->set_new_thread_id();
 
+  DBUG_EXECUTE_IF("bootstrap_crash", DBUG_SUICIDE(););
+  DBUG_EXECUTE_IF("bootstrap_hang", {
+    while (1)
+      my_sleep(1000000);
+  });
+  DBUG_EXECUTE_IF("bootstrap_buffer_overrun", {
+    int *mem = static_cast<int *>(my_malloc(PSI_NOT_INSTRUMENTED, 127, 0));
+    // Allocations are usually aligned, so even if 127 bytes were requested,
+    // it's mostly safe to assume there are 128 bytes. Writing into the last
+    // byte is safe for the rest of the code, but still enough to trigger
+    // AddressSanitizer (ASAN) or Valgrind.
+    my_atomic_store32(mem + (128 / sizeof(*mem)) - 1, 1);
+    my_free(mem);
+  });
+
   bootstrap_file=file;
 
   my_thread_attr_t thr_attr;
