@@ -896,8 +896,8 @@ bool File_query_log::write_slow(THD *thd, ulonglong current_utime,
   if (my_b_printf(&log_file,
                   "# Schema: %s  Last_errno: %u  Killed: %u\n"
                   "# Query_time: %s  Lock_time: %s  Rows_sent: %llu"
-                  "  Rows_examined: %llu  Rows_affected: %llu\n"
-                  "# Bytes_sent: %lu",
+                  "  Rows_examined: %llu  Rows_affected: %llu"
+                  "  Bytes_sent: %lu\n",
                   (thd->db().str ? thd->db().str : ""),
                   thd->last_errno, (uint) thd->killed,
                   query_time_buff, lock_time_buff,
@@ -911,15 +911,12 @@ bool File_query_log::write_slow(THD *thd, ulonglong current_utime,
 
   if (thd->variables.log_slow_verbosity & (1ULL << SLOG_V_QUERY_PLAN))
     if (my_b_printf(&log_file,
-                    "  Tmp_tables: %lu  Tmp_disk_tables: %lu  "
-                    "Tmp_table_sizes: %llu",
+                    "# Tmp_tables: %lu  Tmp_disk_tables: %lu  "
+                    "Tmp_table_sizes: %llu\n",
                     thd->tmp_tables_used, thd->tmp_tables_disk_used,
                     thd->tmp_tables_size) == (uint) -1)
       goto err;
 
-  if (my_b_write(&log_file, (uchar*) "\n", 1))
-    goto err;
-
   if (opt_log_slow_sp_statements == 1 && thd->sp_runtime_ctx &&
       my_b_printf(&log_file,
                   "# Stored_routine: %s\n",
