@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -1283,25 +1283,7 @@ void Slave_worker::slave_worker_ends_group(Log_event* ev, int error)
 
       /* Fatal error happens, it notifies the following transaction to rollback */
       if (get_commit_order_manager())
-      {
-        /*
-          If we still have the deadlock flag set, it means that the current
-          thread was involved in a deadlock in its last retry (or all retries
-          have been exhausted). In such a case, we must release all transaction
-          locks by rolling back the transaction and clear the deadlock flag
-          before we wait for this worker's turn in report_rollback().
-        */
-        if (found_order_commit_deadlock())
-        {
-          /*
-            We call cleanup_context() because it is even capable of rolling
-            back XA transactions.
-          */
-          cleanup_context(info_thd, true);
-          reset_order_commit_deadlock();
-        }
         get_commit_order_manager()->report_rollback(this);
-      }
 
       // Killing Coordinator to indicate eventual consistency error
       mysql_mutex_lock(&c_rli->info_thd->LOCK_thd_data);
@@ -2007,6 +1989,7 @@ bool Slave_worker::retry_transaction(uint start_relay_number,
 {
   THD *thd= info_thd;
   bool silent= false;
+  Slave_worker::Retry_context_sentry cleaned_up(*this);
 
   DBUG_ENTER("Slave_worker::retry_transaction");
 
@@ -2017,6 +2000,7 @@ bool Slave_worker::retry_transaction(uint start_relay_number,
   {
     /* Simulate a lock deadlock error */
     uint error= 0;
+    cleaned_up= false;
 
     if (found_order_commit_deadlock())
     {
@@ -2091,8 +2075,7 @@ bool Slave_worker::retry_transaction(uint start_relay_number,
     c_rli->retried_trans++;
     mysql_mutex_unlock(&c_rli->data_lock);
 
-    cleanup_context(thd, 1);
-    reset_order_commit_deadlock();
+    cleaned_up.clean();
     worker_sleep(min<ulong>(trans_retries, MAX_SLAVE_RETRY_PAUSE));
 
   } while (read_and_apply_events(start_relay_number, start_relay_pos,
@@ -2321,6 +2304,28 @@ static int en_queue(Slave_jobs_queue *jobs, Slave_job_item *item)
   return jobs->avail;
 }
 
+Slave_worker::Retry_context_sentry::Retry_context_sentry(Slave_worker &parent)
+    : m_parent(parent), m_is_cleaned_up(false) {}
+
+Slave_worker::Retry_context_sentry::~Retry_context_sentry() {
+  this->clean();
+}
+
+Slave_worker::Retry_context_sentry &Slave_worker::Retry_context_sentry::operator=(
+    bool is_cleaned_up)
+{
+  this->m_is_cleaned_up = is_cleaned_up;
+  return (*this);
+}
+
+void Slave_worker::Retry_context_sentry::clean() {
+  if (!this->m_is_cleaned_up) {
+    this->m_parent.cleanup_context(this->m_parent.info_thd, 1);
+    this->m_parent.reset_order_commit_deadlock();
+    this->m_is_cleaned_up = true;
+  }
+}
+
 /**
    return the value of @c data member of the head of the queue.
 */
