@@ -1130,6 +1130,10 @@ static int check_connection(THD *thd, bool extra_port_connection)
     char ip[NI_MAXHOST];
     LEX_CSTRING main_sctx_ip;
 
+    if (extra_port_connection) {
+      vio_force_skip_proxy(net->vio);
+    }
+
     peer_rc= vio_peer_addr(net->vio, ip, &thd->peer_port, NI_MAXHOST);
 
     /*
