@@ -2,13 +2,21 @@
 
 Copyright (c) 2013, 2018, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -677,10 +685,14 @@ Datafile::validate_first_page(lsn_t*	flush_lsn,
 	can't be open. And for importing, we skip checking it. */
 	if (FSP_FLAGS_GET_ENCRYPTION(m_flags) && !for_import) { 
 		if(crypt_data == NULL) {
-			m_encryption_key = static_cast<byte*>(
-				ut_zalloc_nokey(ENCRYPTION_KEY_LEN));
-			m_encryption_iv = static_cast<byte*>(
-				ut_zalloc_nokey(ENCRYPTION_KEY_LEN));
+			if (m_encryption_key == NULL) {
+				m_encryption_key = static_cast<byte *>(
+					ut_zalloc_nokey(ENCRYPTION_KEY_LEN));
+			}
+			if (m_encryption_iv == NULL) {
+				m_encryption_iv = static_cast<byte *>(
+					ut_zalloc_nokey(ENCRYPTION_KEY_LEN));
+			}
 #ifdef	UNIV_ENCRYPT_DEBUG
 			fprintf(stderr, "Got from file %lu:", m_space_id);
 #endif
