@@ -2,13 +2,21 @@
 
 Copyright (c) 1997, 2016, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -45,7 +53,7 @@ block.
 bool
 log_block_checksum_is_ok(
 	const byte*	block)	/*!< in: pointer to a log block */
-	__attribute__((warn_unused_result));
+	MY_ATTRIBUTE((warn_unused_result));
 
 /*******************************************************//**
 Calculates the new value for lsn when more data is added to the log. */
@@ -74,7 +82,7 @@ recv_read_checkpoint_info_for_backup(
 	lsn_t*		first_header_lsn)
 				/*!< out: lsn of of the start of the
 				first log file */
-	__attribute__((nonnull));
+	MY_ATTRIBUTE((nonnull));
 /*******************************************************************//**
 Scans the log segment and n_bytes_scanned is set to the length of valid
 log scanned. */
@@ -249,7 +257,7 @@ recv_get_fil_addr_struct(
 /*=====================*/
 	ulint	space,	/*!< in: space id */
 	ulint	page_no)/*!< in: page number */
-	__attribute__((warn_unused_result));
+	MY_ATTRIBUTE((warn_unused_result));
 
 #ifdef UNIV_HOTBACKUP
 /*******************************************************************//**
@@ -313,7 +321,16 @@ struct recv_addr_t{
 
 struct recv_dblwr_t {
 	/** Add a page frame to the doublewrite recovery buffer. */
-	void add(const byte* page) {
+	void add(byte* page) {
+		pages.push_back(page);
+	}
+
+	/** Add a page frame to sys_list and the global list of double
+	write pages. The separate list is used to decrypt doublewrite
+	buffer pages of encrypted system tablespace
+	@param[in]	page	doublwrite buffer page */
+	void add_to_sys(byte* page) {
+		sys_pages.push_back(page);
 		pages.push_back(page);
 	}
 
@@ -324,10 +341,22 @@ struct recv_dblwr_t {
 	@retval NULL if no page was found */
 	const byte* find_page(ulint space_id, ulint page_no);
 
-	typedef std::list<const byte*, ut_allocator<const byte*> >	list;
+	typedef std::list<byte*, ut_allocator<byte*> >	list;
 
 	/** Recovered doublewrite buffer page frames */
 	list	pages;
+
+	/** Pages from system tablespace doublewrite buffer.
+	If encrypted, these pages should be decrypted with system tablespace
+	encryption key. Other pages from parallel double write buffer should
+	be decrypted with their respective tablespace encryption key */
+	list 	sys_pages;
+
+	/** Decrypt double write buffer pages if system tablespace is
+	encrypted. This function process only pages from sys_pages list.
+	Other pages from parallel doublewrite buffer will be decrypted after
+	tablespace objects are loaded. */
+	void decrypt_sys_dblwr_pages();
 };
 
 /* Recovery encryption information */
@@ -405,6 +434,10 @@ struct recv_sys_t{
 
 	encryption_list_t*	/*!< Encryption information list */
 			encryption_list;
+
+	void set_corrupt_log() {
+		found_corrupt_log = true;
+	}
 };
 
 /** The recovery system */
