@@ -68,10 +68,6 @@ ulong		srv_max_purge_lag_delay = 0;
 /** The global data structure coordinating a purge */
 trx_purge_t*	purge_sys = NULL;
 
-/** A dummy undo record used as a return value when we have a whole undo log
-which needs no purge */
-trx_undo_rec_t	trx_purge_dummy_rec;
-
 #ifdef UNIV_DEBUG
 my_bool		srv_purge_view_update_only_debug;
 bool		trx_commit_disallowed = false;
@@ -80,6 +76,10 @@ bool		trx_commit_disallowed = false;
 /** Sentinel value */
 const TrxUndoRsegs TrxUndoRsegsIterator::NullElement(UINT64_UNDEFINED);
 
+/** A sentinel undo record used as a return value when we have a whole
+undo log which can be skipped by purge */
+static trx_undo_rec_t	trx_purge_ignore_rec;
+
 /** Constructor */
 TrxUndoRsegsIterator::TrxUndoRsegsIterator(trx_purge_t* purge_sys)
 	:
@@ -278,6 +278,9 @@ trx_purge_sys_create(
 	purge_sys->view_active = true;
 
 	purge_sys->rseg_iter = UT_NEW_NOKEY(TrxUndoRsegsIterator(purge_sys));
+
+	/* Allocate 8K bytes for the initial heap. */
+	purge_sys->heap = mem_heap_create(8 * 1024);
 }
 
 /************************************************************************
@@ -321,6 +324,10 @@ trx_purge_sys_close(void)
 
 	purge_sys->event = NULL;
 
+	mem_heap_free(purge_sys->heap);
+
+	purge_sys->heap = NULL;
+
 	UT_DELETE(purge_sys->rseg_iter);
 
 	ut_free(purge_sys);
@@ -1500,6 +1507,7 @@ trx_purge_get_next_rec(
 	const page_size_t	page_size(purge_sys->rseg->page_size);
 
 	if (offset == 0) {
+
 		/* It is the dummy undo log record, which means that there is
 		no need to purge this undo log */
 
@@ -1510,7 +1518,7 @@ trx_purge_get_next_rec(
 
 		trx_purge_choose_next_log();
 
-		return(&trx_purge_dummy_rec);
+		return(&trx_purge_ignore_rec);
 	}
 
 	mtr_start(&mtr);
@@ -1598,7 +1606,7 @@ trx_purge_get_next_rec(
 /********************************************************************//**
 Fetches the next undo log record from the history list to purge. It must be
 released with the corresponding release function.
-@return copy of an undo log record or pointer to trx_purge_dummy_rec,
+@return copy of an undo log record or pointer to trx_purge_ignore_rec,
 if the whole undo log can skipped in purge; NULL if none left */
 static MY_ATTRIBUTE((warn_unused_result))
 trx_undo_rec_t*
@@ -1637,27 +1645,31 @@ trx_purge_fetch_next_rec(
 	return(trx_purge_get_next_rec(n_pages_handled, heap));
 }
 
-/*******************************************************************//**
-This function runs a purge batch.
+/** This function runs a purge batch.
+@param[in]	n_purge_threads	number of purge threads
+@param[in,out]	purge_sys	purge instance
+@param[in]	batch_size	no. of pages to purge
 @return number of undo log pages handled in the batch */
 static
 ulint
 trx_purge_attach_undo_recs(
-/*=======================*/
-	ulint		n_purge_threads,/*!< in: number of purge threads */
-	trx_purge_t*	purge_sys,	/*!< in/out: purge instance */
-	ulint		batch_size)	/*!< in: no. of pages to purge */
+	const ulint	n_purge_threads,
+	trx_purge_t*	purge_sys,
+	ulint		batch_size)
 {
 	que_thr_t*	thr;
-	ulint		i = 0;
 	ulint		n_pages_handled = 0;
-	ulint		n_thrs = UT_LIST_GET_LEN(purge_sys->query->thrs);
 
 	ut_a(n_purge_threads > 0);
+	ut_a(n_purge_threads <= SRV_MAX_N_PURGE_THREADS);
 
 	purge_sys->limit = purge_sys->iter;
 
-	/* Debug code to validate some pre-requisites and reset done flag. */
+	que_thr_t*	run_thrs[SRV_MAX_N_PURGE_THREADS];
+
+	/* Validate some pre-requisites and reset done flag. */
+	ulint		i = 0;
+
 	for (thr = UT_LIST_GET_FIRST(purge_sys->query->thrs);
 	     thr != NULL && i < n_purge_threads;
 	     thr = UT_LIST_GET_NEXT(thrs, thr), ++i) {
@@ -1665,40 +1677,36 @@ trx_purge_attach_undo_recs(
 		purge_node_t*		node;
 
 		/* Get the purge node. */
-		node = (purge_node_t*) thr->child;
+		node = static_cast<purge_node_t*>(thr->child);
 
 		ut_a(que_node_get_type(node) == QUE_NODE_PURGE);
-		ut_a(node->undo_recs == NULL);
+		ut_a(node->recs == NULL);
 		ut_a(node->done);
 
-		node->done = FALSE;
+		node->done = false;
+
+		ut_a(!thr->is_active);
+
+		run_thrs[i] = thr;
 	}
 
 	/* There should never be fewer nodes than threads, the inverse
 	however is allowed because we only use purge threads as needed. */
 	ut_a(i == n_purge_threads);
-
-	/* Fetch and parse the UNDO records. The UNDO records are added
-	to a per purge node vector. */
-	thr = UT_LIST_GET_FIRST(purge_sys->query->thrs);
-	ut_a(n_thrs > 0 && thr != NULL);
-
 	ut_ad(trx_purge_check_limit());
 
-	i = 0;
+	mem_heap_t*	heap = purge_sys->heap;
 
-	for (;;) {
-		purge_node_t*		node;
-		trx_purge_rec_t*	purge_rec;
+	mem_heap_empty(heap);
 
-		ut_a(!thr->is_active);
+	typedef std::map<
+	    table_id_t, purge_node_t::Recs *, std::less<table_id_t>,
+	    mem_heap_allocator<std::pair<const table_id_t, purge_node_t::Recs *> > >
+	    GroupBy;
 
-		/* Get the purge node. */
-		node = (purge_node_t*) thr->child;
-		ut_a(que_node_get_type(node) == QUE_NODE_PURGE);
+	GroupBy group_by((GroupBy::key_compare()), mem_heap_allocator<GroupBy::value_type>(heap));
 
-		purge_rec = static_cast<trx_purge_rec_t*>(
-			mem_heap_zalloc(node->heap, sizeof(*purge_rec)));
+	for (ulint i = 0; n_pages_handled < batch_size; ++i) {
 
 		/* Track the max {trx_id, undo_no} for truncating the
 		UNDO logs once we have purged the records. */
@@ -1707,38 +1715,75 @@ trx_purge_attach_undo_recs(
 			purge_sys->limit = purge_sys->iter;
 		}
 
+		purge_node_t::rec_t	rec;
+
 		/* Fetch the next record, and advance the purge_sys->iter. */
-		purge_rec->undo_rec = trx_purge_fetch_next_rec(
-			&purge_rec->roll_ptr, &n_pages_handled, node->heap);
+		rec.undo_rec = trx_purge_fetch_next_rec(
+			&rec.roll_ptr, &n_pages_handled, heap);
 
-		if (purge_rec->undo_rec != NULL) {
+		if (rec.undo_rec == &trx_purge_ignore_rec) {
 
-			if (node->undo_recs == NULL) {
-				node->undo_recs = ib_vector_create(
-					ib_heap_allocator_create(node->heap),
-					sizeof(trx_purge_rec_t),
-					batch_size);
-			} else {
-				ut_a(!ib_vector_is_empty(node->undo_recs));
-			}
+			continue;
 
-			ib_vector_push(node->undo_recs, purge_rec);
+		} else if (rec.undo_rec == NULL) {
 
-			if (n_pages_handled >= batch_size) {
-
-				break;
-			}
-		} else {
 			break;
 		}
 
-		thr = UT_LIST_GET_NEXT(thrs, thr);
+		table_id_t	table_id;
 
-		if (!(++i % n_purge_threads)) {
-			thr = UT_LIST_GET_FIRST(purge_sys->query->thrs);
+		table_id = trx_undo_rec_get_table_id(rec.undo_rec);
+
+		GroupBy::iterator lb = group_by.lower_bound(table_id);
+
+		if (lb != group_by.end()
+		    && !(group_by.key_comp()(table_id, lb->first))) {
+
+			lb->second->push_back(rec);
+
+		} else {
+			typedef GroupBy::value_type value_type;
+
+			void*			ptr;
+			purge_node_t::Recs*	recs;
+
+			ptr = mem_heap_alloc(heap, sizeof(purge_node_t::Recs));
+
+			/* Call the destructor explicitly in row_purge_end() */
+			recs = new (ptr) purge_node_t::Recs(mem_heap_allocator<purge_node_t::rec_t>(heap));
+
+			recs->push_back(rec);
+
+			group_by.insert(lb, value_type(table_id, recs));
 		}
+	}
 
-		ut_a(thr != NULL);
+	/* Objective is to ensure that all the table entries in one
+	batch are handled by the same thread. Ths is to avoid contention
+	on the dict_index_t::lock */
+
+	const GroupBy& group_by_const = group_by;
+	GroupBy::const_iterator	end = group_by_const.end();
+
+	for (GroupBy::const_iterator it = group_by_const.begin(); it != end; ) {
+
+		for (ulint i = 0; i < n_purge_threads && it != end; ++i, ++it) {
+
+			purge_node_t*	node;
+
+			node = static_cast<purge_node_t*>(run_thrs[i]->child);
+
+			ut_a(que_node_get_type(node) == QUE_NODE_PURGE);
+
+			if (node->recs == NULL) {
+				node->recs = it->second;
+			} else {
+				node->recs->insert(
+					node->recs->end(),
+					it->second->begin(),
+					it->second->end());
+			}
+		}
 	}
 
 	ut_ad(trx_purge_check_limit());
@@ -1796,17 +1841,24 @@ trx_purge_wait_for_workers_to_complete(
 /*===================================*/
 	trx_purge_t*	purge_sys)	/*!< in: purge instance */
 {
+	ulint		i = 0;
 	ulint		n_submitted = purge_sys->n_submitted;
 
 	/* Ensure that the work queue empties out. */
 	while (!os_compare_and_swap_ulint(
 			&purge_sys->n_completed, n_submitted, n_submitted)) {
 
-		if (srv_get_task_queue_length() > 0) {
-			srv_release_threads(SRV_WORKER, 1);
-		}
+		if (++i < 10) {
+			os_thread_yield();
+		} else {
 
-		os_thread_yield();
+			if (srv_get_task_queue_length() > 0) {
+				srv_release_threads(SRV_WORKER, 1);
+			}
+
+			os_thread_sleep(20);
+			i = 0;
+		}
 	}
 
 	/* None of the worker threads should be doing any work. */
@@ -1877,10 +1929,9 @@ trx_purge(
 
 	/* Do we do an asynchronous purge or not ? */
 	if (n_purge_threads > 1) {
-		ulint	i = 0;
 
 		/* Submit the tasks to the work queue. */
-		for (i = 0; i < n_purge_threads - 1; ++i) {
+		for (ulint i = 0; i < n_purge_threads - 1; ++i) {
 			thr = que_fork_scheduler_round_robin(
 				purge_sys->query, thr);
 
